from attr import dataclass


@dataclass
class BridgeVlan:
    bridge: str
    current_tagged: str
    current_untagged: str
    disabled: bool
    dynamic: bool
    tagged: str
    untagged: str
    vlan_ids: int
    comment: str = None
    id: str = None

    def __bool__(self) -> bool:
        return not self.disabled



from typing import Literal


IPv4Options = Literal[
    "any",
    "loose_source_routing",
    "no_record_route",
    "no_router_alert",
    "no_source_routing",
    "no_timestamp",
    "none",
    "record_route",
    "router_alert",
    "strict_source_routing",
    "timestamp",
]



from attr import dataclass
from typing import Literal

from ros._literals import ARPLiteral

LoopProtectLiteral = Literal["default", "on", "off"]


@dataclass
class Vlan:
    name: str
    vlan_id: int
    interface: str
    arp: ARPLiteral = "enabled"
    mtu: int = 1500
    arp_timeout: str = None
    use_service_tag: bool = False
    loop_protect: LoopProtectLiteral = "default"
    loop_protect_disable_time: str = None
    loop_protect_send_interval: str = None
    comment: str = None
    disabled: bool = False
    copy_from: str = None
    id: str = None
    mac_address: str = None
    l2mtu: int = None
    loop_protect_status: str = None
    running: bool = None

    def __str__(self) -> str:
        return self.name



from attr import dataclass


@dataclass
class MPLSSettings:
    dynamic_label_range: str
    propagate_ttl: bool
    allow_fast_path: bool
    mpls_fast_path_packets: int
    mpls_fast_path_bytes: int



from attr import dataclass
from typing import Union


@dataclass
class Nexthop:
    id: str
    address: str
    afi: str
    scope: int
    target_scope: int
    gw_check_ok: bool
    interface_ok: bool
    bgp_vpn: bool
    immediate_gwweight: Union[str, int]
    immediate_gwflap_count: Union[str, int]
    immediate_gwinterface_idx: Union[str, int]
    immediate_gwmpls_peer_id: Union[str, int]
    immediate_gwmpls_label: Union[str, int]
    immediate_gwblackhole: bool
    reachable: bool
    unresolved: bool
    immediate_gw_address: str = None
    check_gateway: str = None



from typing import List, Literal, Union

from ros._base import BaseModule, BaseProps
from ros._literals import AnyLiteral, IPProtocol, MACProtocol, PortLiteral

from .bandwith_server import BandwithServer
from .bandwith_test import BandwithTest
from .ip_scan import IPScan
from .netwatch import Netwatch
from .ping import Ping
from .profile import Profile
from .torch import Torch
from .traceroute import Traceroute


class ToolModule(BaseModule):
    _netwatch: BaseProps[Netwatch] = None

    @property
    def bandwith_server(self) -> BandwithServer:
        return self.ros.get_as(self.url + "/bandwidth-server", BandwithServer)

    def ping(
        self,
        address: str,
        count: int = 4,
        interface: str = None,
        arp_ping: bool = False,
        src_address: str = None,
        size: int = 56,
        dscp: int = 0,
        ttl: int = 0,
        vrf: str = "main",
    ):
        data = {
            "address": address,
            "count": count,
            "arp-ping": arp_ping,
            "size": size,
            "dscp": dscp,
            "vrf": vrf,
        }
        if interface:
            data["interface"] = interface
        if src_address:
            data["src-address"] = src_address
        if ttl > 0:
            data["ttl"] = ttl
        return self.ros.post_as("/ping", List[Ping], data)

    def profile(
        self,
        cpu: Literal["total", "all"] = "total",
        duration: str = "5s",
    ) -> List[Profile]:
        data = {"cpu": cpu, "duration": duration}
        return self.ros.post_as("/tool/profile", List[Profile], data)

    def bandwith_test(
        self,
        address: str,
        duration: str,
        user: str = None,
        password: str = None,
        protocol: Literal["tcp", "udp"] = "udp",
        direction: Literal["receive", "send", "both"] = "receive",
        random_data: bool = False,
    ) -> List[BandwithTest]:
        data = {
            "address": address,
            "duration": duration,
            "user": user,
            "password": password,
            "protocol": protocol,
            "direction": direction,
            "random-data": random_data,
        }
        return self.ros.post_as(self.url + "/bandwidth-test", List[BandwithTest], data)

    def ip_scan(
        self,
        interface: str,
        duration: str,
        address_range: str = None,
        freeze_frame_interval: str = None,
    ) -> List[IPScan]:
        data = {"interface": interface, "duration": duration}
        if address_range:
            data["address-range"] = address_range
        if freeze_frame_interval:
            data["freeze-frame-interval"] = freeze_frame_interval
        return self.ros.post_as(self.url + "/ip-scan", List[IPScan], data)

    @property
    def netwatch(self):
        if not self._netwatch:
            self._netwatch = BaseProps(self.ros, "/tool/netwatch", Netwatch)
        return self._netwatch

    def torch(
        self,
        interface: str,
        duration: str = "5s",
        src_address: str = "0.0.0.0/0",
        dst_address: str = "0.0.0.0/0",
        src_address6: str = "::/0",
        dst_address6: str = "::/0",
        mac_protocol: Union[AnyLiteral, MACProtocol] = None,
        port: Union[AnyLiteral, PortLiteral] = None,
        ip_protocol: Union[AnyLiteral, IPProtocol] = None,
        vlan_id: Union[AnyLiteral, int] = None,
        dscp: Union[AnyLiteral, int] = None,
        cpu: str = None,
        freeze_frame_interval: str = None,
    ):
        data = {"interface": interface, "duration": duration}
        if src_address:
            data["src-address"] = src_address
        if dst_address:
            data["dst-address"] = dst_address
        if src_address6:
            data["src-address6"] = src_address6
        if dst_address6:
            data["dst-address6"] = dst_address6
        if mac_protocol:
            data["mac-protocol"] = mac_protocol
        if port:
            data["port"] = port
        if ip_protocol:
            data["ip-protocol"] = ip_protocol
        if vlan_id:
            data["vlan-id"] = vlan_id
        if dscp:
            data["dscp"] = dscp
        if cpu:
            data["cpu"] = cpu
        if freeze_frame_interval:
            data["freeze-frame-interval"] = freeze_frame_interval
        return self.ros.post_as(self.url + "/torch", List[Torch], data)

    def traceroute(
        self,
        address: str,
        duration: str = "5s",
        size: int = 56,
        timeout: str = "1000ms",
        protocol: str = "icmp",
        port: int = "33434",
        use_dns: bool = False,
        count: int = None,
        max_hops: int = None,
        src_address: str = None,
        interface: str = None,
        dscp: int = 0,
        vrf: str = "main",
    ) -> List[Traceroute]:
        data = {
            "address": address,
            "duration": duration,
            "size": size,
            "timeout": timeout,
            "protocol": protocol,
            "port": port,
            "use-dns": use_dns,
            "dscp": dscp,
            "vrf": vrf,
        }
        if count and count > 0:
            data["count"] = count
        if max_hops and max_hops > 0:
            data["max-hops"] = max_hops
        if interface:
            data["interface"] = interface
        if src_address:
            data["src-address"] = src_address
        return self.ros.post_as(self.url + "/traceroute", List[Traceroute], data)

    def torch(
        self,
        interface: str,
        duration: str = "3s",
        src_address: str = "0.0.0.0/0",
        dst_address: str = "0.0.0.0/0",
        src_address6: str = "::/0",
        dst_address6: str = "::/0",
        mac_protocol: Union[AnyLiteral, MACProtocol] = "any",
        ip_protocol: Union[AnyLiteral, IPProtocol] = "any",
        port: PortLiteral = "any",
        vlan_id: Union[AnyLiteral, int] = "any",
        dscp: Union[AnyLiteral, int] = "any",
        cpu: Union[AnyLiteral, int] = "any",
        freeze_frame_interval: str = None,
    ) -> List[Torch]:
        data = {
            "interface": interface,
            "dst-address": dst_address,
            "mac-protocol": mac_protocol,
            "src-address6": src_address6,
            "cpu": cpu,
            "dst-address6": dst_address6,
            "port": port,
            "vlan-id": vlan_id,
            "dscp": dscp,
            "duration": duration,
            "ip-protocol": ip_protocol,
            "src-address": src_address,
        }
        if freeze_frame_interval:
            data["freeze-frame-interval"] = freeze_frame_interval
        return self.ros.post_as(self.url + "/torch", List[Torch], data)

    def wol(self, interface: str, mac: str) -> List[dict]:
        data = {"interface": interface, "mac": mac}
        return self.ros.post_as(self.url + "/wol", List[dict], data)


__all__ = [
    "BandwithServer",
    "BandwithTest",
    "IPScan",
    "Ping",
    "ToolModule",
    "Torch",
    "Traceroute",
]



from attr import dataclass


@dataclass(slots=True)
class BandwithServer:
    allocate_udp_ports_from: int
    authenticate: bool
    enabled: bool
    max_sessions: int

    def __bool__(self):
        return self.enabled



from attr import dataclass, field


@dataclass
class DHCPRelay:
    name: str
    interface: str
    dhcp_server: str
    delay_threshold: str = None
    local_address: str = None
    add_relay_info: bool = False
    disabled: bool = False
    invalid: bool = field(on_setattr=None, default=None)
    id: str = field(on_setattr=None, default=None)

    def __str__(self) -> str:
        return self.name

    def __bool__(self) -> bool:
        return not self.disabled



from attr import dataclass


@dataclass
class Vrf:
    name: str
    interfaces: str
    comment: str = None
    disabled: bool = False
    id: str = None
    builtin: bool = None
    copy_from: str = None
    place_before: str = None

    def __str__(self) -> str:
        return self.name



from attr import dataclass
from typing import Literal, Union
from ros._base import BaseProps

QueuePriority = Literal["0", "1", "2", "3", "4", "5", "6", "7", "8"]


@dataclass
class QueueTree:
    name: str
    parent: str
    packet_mark: str
    queue: str
    priority: QueuePriority = "8"
    bucket_size: float = 0.100
    limit_at: str = None
    max_limit: str = None
    burst_limit: str = None
    burst_threshold: str = None
    burst_time: str = None
    disabled: bool = None
    invalid: bool = None
    bytes: int = None
    dropped: int = None
    queued_bytes: int = None
    queued_packets: int = None
    rate: int = None
    comment: str = None
    id: str = None
    copy_from: str = None

    def __str__(self) -> str:
        return self.name


class QueueTreeModule(BaseProps[QueueTree]):
    def reset(self, queue: Union[QueueTree, str]):
        rid = queue.id if isinstance(queue, QueueTree) else queue
        self.ros.post_as("/queue/tree/reset", None, {".id": rid})

    def reset_counters(self, queue: Union[QueueTree, str]):
        rid = queue.id if isinstance(queue, QueueTree) else queue
        self.ros.post_as("/queue/tree/reset-counters", None, {".id": rid})

    def reset_counters_all(self):
        self.ros.post_as("/queue/tree/reset-counters-all", None)



from ros._base import BaseProps
from .active import HotspotActive
from .cookie import HotspotCookie
from .host import HotspotHost
from .hotspot import HotspotServer
from .ip_binding import HotspotIPBinding
from .profile import HotspotProfile
from .service_port import HotspotServicePort
from .user import HotspotUserModule, HotspotUser
from .walled_garden import HotspotWalledGardenModule, HotspotWalledGarden


class HotspotModule(BaseProps[HotspotServer]):
    _active: BaseProps[HotspotActive] = None
    _cookie: BaseProps[HotspotCookie] = None
    _host: BaseProps[HotspotHost] = None
    _ip_binding: BaseProps[HotspotIPBinding] = None
    _user: HotspotUserModule = None
    _profile: BaseProps[HotspotProfile] = None
    _service_port: BaseProps[HotspotServicePort] = None
    _walled_garden: HotspotWalledGardenModule = None

    @property
    def active(self) -> BaseProps[HotspotActive]:
        if not self._active:
            self._active = BaseProps(self.ros, "/ip/hotspot/active", HotspotActive)
            self._active._create = False
            self._active._write = False
        return self._active

    @property
    def cookie(self) -> BaseProps[HotspotCookie]:
        if not self._cookie:
            self._cookie = BaseProps(self.ros, "/ip/hotspot/cookie", HotspotCookie)
            self._cookie._create = False
            self._cookie._write = False
        return self._cookie

    @property
    def host(self) -> BaseProps[HotspotHost]:
        if not self._host:
            self._host = BaseProps(self.ros, "/ip/hotspot/host", HotspotHost)
            self._host._create = False
            self._host._write = False
        return self._host

    @property
    def ip_binding(self) -> BaseProps[HotspotIPBinding]:
        if not self._ip_binding:
            self._ip_binding = BaseProps(
                self.ros, "/ip/hotspot/ip-binding", HotspotIPBinding
            )
        return self._ip_binding

    @property
    def user(self) -> HotspotUserModule:
        if not self._user:
            self._user = HotspotUserModule(self.ros, "/ip/hotspot/user", HotspotUser)
        return self._user

    @property
    def profile(self) -> BaseProps[HotspotProfile]:
        if not self._profile:
            self._profile = BaseProps(self.ros, "/ip/hotspot/profile", HotspotProfile)
        return self._profile

    @property
    def service_port(self) -> BaseProps[HotspotServicePort]:
        if not self._service_port:
            self._service_port = BaseProps(
                self.ros, "/ip/hotspot/service-port", HotspotServicePort
            )
            self._service_port._delete = False
        return self._service_port

    @property
    def walled_garden(self) -> HotspotWalledGardenModule:
        if not self._walled_garden:
            self._walled_garden = HotspotWalledGardenModule(
                self.ros, "/ip/hotspot/walled-garden", HotspotWalledGarden
            )
        return self._walled_garden


__all__ = [
    "HotspotActive",
    "HotspotHost",
    "HotspotIPBinding",
    "HotspotProfile",
    "HotspotServer",
    "HotspotServicePort",
    "HotspotUser",
    "HotspotWalledGarden",
    "HotspotModule",
]



from attr import dataclass


@dataclass
class IPPool:
    name: str
    ranges: str
    next_pool: str = None
    comment: str = None
    copy_from: str = None
    id: str = None

    def __str__(self) -> str:
        return self.name



from attr import dataclass


@dataclass
class IPNeighbor:
    id: str
    interface: str
    address: str
    address4: str
    mac_address: str
    identity: str
    platform: str
    version: str
    age: str
    uptime: str
    software_id: str
    board: str
    interface_name: str
    discovered_by: str
    system_description: str = None
    system_caps: str = None
    system_caps_enabled: str = None
    address6: str = None
    unpack: str = None



from attr import dataclass

from ros._literals import ARPLiteral


@dataclass
class Bridge:
    name: str
    arp: ARPLiteral
    ageing_time: str
    arp_timeout: str
    auto_mac: bool
    dhcp_snooping: bool
    disabled: bool
    fast_forward: bool
    forward_delay: str
    igmp_snooping: bool
    mac_address: str
    max_message_age: str
    mtu: str
    priority: str
    protocol_mode: str
    running: bool
    transmit_hold_count: int
    vlan_filtering: bool
    actual_mtu: int = None
    l2mtu: int = None
    comment: str = None
    id: str = None

    def __str__(self) -> str:
        return self.name

    def __bool__(self) -> bool:
        return not self.disabled



from attr import dataclass
from typing import Literal

from ._literals import IPv4Options

Action = Literal[
    "accept",
    "add-dst-to-address-list",
    "add-src-to-address-list",
    "dst-nat",
    "endpoint-independent-nat",
    "jump",
    "log",
    "masquerade",
    "netmap",
    "passthrough",
    "redirect",
    "return",
    "same",
    "src-nat",
]


@dataclass
class IPFirewallNAT:
    # General
    chain: str
    src_address: str = None
    dst_address: str = None
    src_address_list: str = None
    dst_address_list: str = None
    protocol: str = None
    src_port: str = None
    dst_port: str = None
    port: str = None
    in_interface: str = None
    out_interface: str = None
    in_interface_list: str = None
    out_interface_list: str = None
    packet_mark: str = None
    connection_mark: str = None
    routing_mark: str = None
    connection_type: str = None
    disabled: bool = None
    comment: str = None
    # Advanced
    layer7_protocol: str = None
    content: str = None
    connection_bytes: str = None
    connection_rate: str = None
    per_connection_classifier: str = None
    src_mac_address: str = None
    in_bridge_port: str = None
    out_bridge_port: str = None
    in_bridge_port_list: str = None
    out_bridge_port_list: str = None
    ipsec_policy: str = None
    ingress_priority: str = None
    priority: str = None
    dscp: str = None
    tcp_mss: str = None
    packet_size: str = None
    random: str = None
    icmp_options: str = None
    ipv4_options: IPv4Options = None
    ttl: str = None
    # Extra
    nth: str = None
    connection_limit: str = None
    src_address_type: str = None
    dst_address_type: str = None
    hotspot: str = None
    fragment: bool = None
    limit: str = None
    dst_limit: str = None
    time: str = None
    psd: str = None
    # Action
    action: Action = "accept"
    log: bool = False
    log_prefix: str = None
    address_list: str = None
    address_list_timeout: str = None
    to_addresses: str = None
    to_ports: str = None
    jump_target: str = None
    same_not_by_dst: bool = None
    # Statistics
    bytes: int = None
    packets: int = None
    # Etc
    id: str = None
    copy_from: str = None
    dynamic: bool = None
    invalid: bool = None
    place_before: str = None
    realm: str = None
    tls_host: str = None

    def __bool__(self) -> bool:
        return not self.disabled



from attr import dataclass

from ._literals import YesNoDefault, YesNoDefaultRequired


@dataclass
class PPPProfile:
    id: str
    address_list: str
    bridge_learning: YesNoDefault
    change_tcp_mss: YesNoDefault
    default: bool
    name: str
    on_down: str
    on_up: str
    only_one: YesNoDefault
    use_compression: YesNoDefault
    use_encryption: YesNoDefaultRequired
    use_ipv6: YesNoDefaultRequired
    use_mpls: YesNoDefaultRequired
    use_upnp: YesNoDefault
    bridge: str = None
    bridge_horizon: int = None
    bridge_path_cost: int = None
    bridge_port_priority: str = None
    comment: str = None
    dhcpv6_pd_pool: str = None
    dns_server: str = None
    idle_timeout: str = None
    incoming_filter: str = None
    insert_queue_before: str = None
    interface_list: str = None
    local_address: str = None
    outgoing_filter: str = None
    parent_queue: str = None
    queue_type: str = None
    rate_limit: str = None
    remote_address: str = None
    remote_ipv6_prefix_pool: str = None
    session_timeout: str = None
    wins_server: str = None

    def __str__(self) -> str:
        return self.name



from ros._base import BaseModule, BaseProps

from .configuration import BFDConfiguration
from .session import BFDSession


class BFDModule(BaseModule):
    _configuration: BaseProps[BFDConfiguration] = None
    _session: BaseProps[BFDSession] = None

    @property
    def configuration(self) -> BaseProps[BFDConfiguration]:
        if not self._configuration:
            self._configuration = BaseProps(
                self.ros, "/routing/bfd/configuration", BFDConfiguration
            )
        return self._configuration

    @property
    def session(self) -> BaseProps[BFDSession]:
        if not self._session:
            self._session = BaseProps(self.ros, "/routing/bfd/session", BFDSession)
        return self._session


__all__ = ["BFDConfiguration", "BFDSession", "BFDModule"]



from attr import dataclass


@dataclass(slots=True)
class BandwithTest:
    section: int
    connection_count: int
    direction: str
    lost_packets: int
    random_data: bool
    status: str
    rx_10_second_average: int = None
    rx_current: int = None
    rx_size: int = None
    rx_total_average: int = None
    tx_10_second_average: int = None
    tx_current: int = None
    tx_size: int = None
    tx_total_average: int = None
    about: str = None
    duration: str = None
    local_cpu_load: int = None
    remote_cpu_load: int = None



from attr import dataclass


@dataclass(slots=True)
class Ping:
    avg_rtt: str
    host: str
    max_rtt: str
    min_rtt: str
    packet_loss: int
    received: int
    sent: int
    seq: int
    size: int
    time: str
    ttl: int



from attr import dataclass
from typing import Literal

MacFormatLiteral = Literal[
    "XX XX XX XX XX XX",
    "XX-XX-XX-XX-XX-XX",
    "XX:XX:XX:XX:XX:XX",
    "XXXX:XXXX:XXXX",
    "XXXXXX-XXXXXX",
    "XXXXXX:XXXXXX",
    "XXXXXXXXXXXX",
]
NASPortLiteral = Literal["cable", "ethernet", "wireless-802.11"]


@dataclass
class HotspotProfile:
    name: str
    hotspot_address: str
    dns_name: str
    html_directory: str
    html_directory_override: str = None
    install_hotspot_queue: bool = True
    rate_limit: str = None
    http_proxy: str = None
    smtp_server: str = None
    login_by: str = None
    mac_auth_mode: str = None
    mac_auth_password: str = None
    http_cookie_lifetime: str = "3d 00:00:00"
    ssl_certificate: str = "none"
    split_user_domain: bool = False
    trial_uptime_limit: str = None
    trial_uptime_reset: str = None
    trial_user_profile: str = None
    use_radius: bool = False
    radius_default_domain: str = None
    radius_location_id: str = None
    radius_location_name: str = None
    radius_mac_format: MacFormatLiteral = "XX:XX:XX:XX:XX:XX"
    radius_accounting: bool = True
    radius_interim_update: str = None
    nas_port_type: NASPortLiteral = "wireless-802.11"
    copy_from: str = None
    id: str = None

    def __str__(self) -> str:
        return self.name



from attr import dataclass


@dataclass
class LDPLocalMapping:
    vrf: str = None
    dst_address: str = None
    label: str = None
    comment: str = None
    copy_from: str = None
    disabled: bool = None
    dynamic: bool = None
    egress: bool = None
    gateway: bool = None
    id: str = None
    adv_path: str = None
    peers: str = None
    pw_fec: str = None
    vpls: bool = None



from attr import dataclass
from typing import Literal, Union

OpenStatusPageLiteral = Literal["always", "http-login"]


@dataclass
class HotspotUserProfile:
    name: str
    address_pool: str = "none"
    session_timeout: str = None
    idle_timeout: str = "none"
    keepalive_timeout: str = "00:02:00"
    status_autorefresh: str = "00:01:00"
    shared_users: Union[int, str] = 1
    rate_limit: str = None
    add_mac_cookie: bool = True
    mac_cookie_timeout: str = "3d 00:00:00"
    address_list: str = None
    incoming_filter: str = None
    outgoing_filter: str = None
    incoming_packet_mark: str = None
    outgoing_packet_mark: str = None
    open_status_page: OpenStatusPageLiteral = "always"
    transparent_proxy: bool = False
    insert_queue_before: str = None
    parent_queue: str = None
    queue_type: str = None
    on_login: str = None
    on_logout: str = None
    advertise: bool = None
    advertise_url: str = None
    advertise_interval: str = None
    advertise_timeout: str = None
    copy_from: str = None
    id: str = None

    def __str__(self):
        return self.name



from attr import dataclass
from typing import Literal

StatusLiteral = Literal["up", "down", "unknown"]
TypeLiteral = Literal["http-get", "https-get", "icmp", "simple", "tcp-conn"]


@dataclass
class Netwatch:
    host: str
    type: TypeLiteral
    interval: str = None
    timeout: str = None
    start_delay: str = None
    down_script: str = None
    up_script: str = None
    test_script: str = None
    disabled: bool = False
    http_codes: str = None
    port: int = None
    packet_interval: str = None
    packet_count: int = None
    packet_size: int = None
    certificate: str = None
    check_certificate: bool = None
    thr_avg: str = None
    thr_http_time: str = None
    thr_jitter: str = None
    thr_loss_count: str = None
    thr_loss_percent: str = None
    thr_max: str = None
    thr_stdev: str = None
    thr_tcp_conn_time: str = None
    comment: str = None
    status: StatusLiteral = None
    id: str = None
    done_test: int = None
    failed_tests: int = None
    since: str = None
    copy_from: str = None



from typing import Literal

YesNoDefault = Literal["yes", "no", "default"]
YesNoDefaultRequired = Literal["yes", "no", "default", "required"]
PPPService = Literal["any", "async", "l2tp", "ovpn", "pppoe", "pptp", "sstp"]



from attr import dataclass


@dataclass
class UserSSHKey:
    user: str
    bits: str
    key_owner: str
    RSA: bool
    id: str

    def __str__(self) -> str:
        return self.key_owner or self.user



from attr import dataclass
from typing import Literal

TypeLiteral = Literal["blocked", "bypassed", "regular"]


@dataclass
class HotspotIPBinding:
    mac_address: str = None
    address: str = None
    to_address: str = None
    server: str = "all"
    type: TypeLiteral = "regular"
    comment: str = None
    disabled: bool = False
    place_before: str = None
    copy_from: str = None
    id: str = None
    bypassed: bool = False
    blocked: bool = False

    def __attrs_post_init__(self) -> None:
        if not self.mac_address and not self.address:
            raise ValueError("failure: address or mac-address is required")



from attr import dataclass


@dataclass
class Device:
    name: str
    mac_address: str
    user: str
    disabled: bool = None
    copy_from: str = None
    id: str = None
    activity: str = None
    blocked: bool = None
    bytes_down: int = None
    bytes_up: int = None
    dynamic: bool = None
    idle_time: str = None
    inactive: bool = None
    ip_address: str = None
    limited: bool = None
    rate_down: int = None
    rate_up: int = None

    def __str__(self) -> str:
        return self.name



from ros._base import BaseProps
from .device import Device
from .kid import KidControl


class KidControlModule(BaseProps[KidControl]):
    _device: BaseProps[Device] = None

    @property
    def device(self) -> BaseProps[Device]:
        if not self._device:
            self._device = BaseProps(self.ros, "/ip/kid-control/device", Device)
        return self._device


__all__ = ["Device", "KidControl", "KidControlModule"]



from attr import dataclass
from typing import Literal, Union
from ros._literals import ARPLiteral


LoopProtectLiteral = Literal["on", "off", "default"]
DontFragmentLiteral = Literal["inherit", "no"]
MTULiteral = Union[str, int]


@dataclass
class EoIP:
    name: str
    remote_address: str = "0.0.0.0"
    tunnel_id = 0
    arp: ARPLiteral = "enabled"
    arp_timeout: str = None
    allow_fast_path: bool = True
    clamp_tcp_mss: bool = True
    disabled: bool = False
    dont_fragment: DontFragmentLiteral = "no"
    dscp: str = "inherit"
    ipsec_secret: str = None
    keepalive: str = "10s,10"
    local_address: str = None
    loop_protect: LoopProtectLiteral = "default"
    loop_protect_disable_time: str = "00:05:00"
    loop_protect_send_interval: str = "00:00:05"
    mac_address: str = None
    mtu: MTULiteral = None
    comment: str = None
    id: str = None
    l2mtu: int = None
    copy_from: str = None
    running: bool = False

    def __str__(self) -> str:
        return self.name



from attr import dataclass


@dataclass
class InterfaceListMember:
    interface: str
    list: str
    disabled: bool
    dynamic: bool
    comment: str = None
    id: str = None

    def __bool__(self) -> bool:
        return not self.disabled



from attr import dataclass


@dataclass
class HotspotActive:
    id: str
    server: str
    user: str
    address: str
    mac_address: str
    login_by: str
    uptime: str
    keepalive_timeout: str
    session_time_left: str = None
    comment: str = None
    radius: bool = False
    bytes_in: int = 0
    bytes_out: int = 0
    packets_in: int = 0
    packets_out: int = 0
    idle_time: str = None

    def __str__(self) -> str:
        return self.user



from attr import dataclass


@dataclass
class LDPNeighbor:
    transport: str
    sending_targeted_hello: bool = None
    comment: str = None
    copy_from: str = None
    disabled: bool = None
    id: str = None
    addresses: str = None
    dynamic: bool = None
    inactive: bool = None
    local_transport: str = None
    on_demand: bool = None
    operational: bool = None
    passive: bool = None
    path_vector_limit: int = None
    peer: str = None
    used_afi: str = None
    vpls: bool = None



from attrs import define

from ros._base import BaseProps

from .bridge import Bridge
from .msti import BridgeMsti
from .port import BridgePort
from .vlan import BridgeVlan


@define
class BridgeModule(BaseProps[Bridge]):
    _msti: BaseProps[BridgeMsti] = None
    _port: BaseProps[BridgePort] = None
    _vlan: BaseProps[BridgeVlan] = None

    @property
    def msti(self) -> BaseProps[BridgeMsti]:
        if not self._msti:
            self._msti = BaseProps(self.ros, "/interface/bridge/msti", BridgeMsti)
        return self._msti

    @property
    def port(self) -> BaseProps[BridgePort]:
        if not self._port:
            self._port = BaseProps(self.ros, "/interface/bridge/port", BridgePort)
        return self._port

    @property
    def vlan(self) -> BaseProps[BridgeVlan]:
        if not self._vlan:
            self._vlan = BaseProps(self.ros, "/interface/bridge/vlan", BridgeVlan)
        return self._vlan


__all__ = ["BridgeModule", "BridgeInterface", "BridgePort", "Bridge"]



from attr import dataclass


@dataclass
class Setting:
    accept_redirects: bool
    accept_source_route: bool
    allow_fast_path: bool
    arp_timeout: str
    icmp_rate_limit: int
    icmp_rate_mask: str
    ip_forward: bool
    ipv4_fast_path_active: bool
    ipv4_fast_path_bytes: int
    ipv4_fast_path_packets: int
    ipv4_fasttrack_active: bool
    ipv4_fasttrack_bytes: int
    ipv4_fasttrack_packets: int
    max_neighbor_entries: int
    route_cache: bool
    rp_filter: str
    secure_redirects: bool
    send_redirects: bool
    tcp_syncookies: bool



from attr import dataclass


@dataclass(slots=True)
class History:
    id: str
    action: str
    by: str
    policy: str
    redo: str
    undoable: bool

    def __str__(self) -> str:
        return self.action



from attr import dataclass


@dataclass
class BridgeMsti:
    bridge: str
    disabled: bool
    identifier: int
    priority: str
    vlan_mapping: int
    comment: str = None
    id: str = None

    def __bool__(self) -> bool:
        return not self.disabled



from attr import define

from ros._base import BaseProps

from .list import InterfaceList
from .member import InterfaceListMember


@define
class InterfaceListModule(BaseProps[InterfaceList]):
    _member: BaseProps[InterfaceListMember] = None

    @property
    def member(self) -> BaseProps[InterfaceListMember]:
        if not self._member:
            self._member = BaseProps(
                self.ros, "/interface/list/member", InterfaceListMember
            )
        return self._member


__all__ = ["InterfaceList", "InterfaceListMember", "InterfaceListModule"]



from attr import dataclass
from typing import Literal

from ros._base import BaseProps
from .ip import HotspotWalledGardenIP

ActionLiteral = Literal["allow", "deny"]


@dataclass
class HotspotWalledGarden:
    action: ActionLiteral
    server: str = None
    src_address: str = None
    method: str = None
    dst_host: str = None
    dst_port: str = None
    path: str = None
    comment: str = None
    disabled: bool = False
    hits: int = None
    id: str = None
    copy_from: str = None
    place_before: str = None
    dst_address: str = None


class HotspotWalledGardenModule(BaseProps[HotspotWalledGarden]):
    _ip: BaseProps[HotspotWalledGardenIP] = None

    @property
    def ip(self) -> BaseProps[HotspotWalledGardenIP]:
        if not self._ip:
            self._ip = BaseProps(
                self.ros, "/ip/hotspot/walled-garden/ip", HotspotWalledGardenIP
            )
        return self._ip


__all__ = ["HotspotWalledGarden", "HotspotWalledGarden", "HotspotWalledGardenIP"]



from attr import dataclass, field


@dataclass
class DHCPLease:
    # General
    address: str
    mac_address: str = None
    use_src_mac: bool = False
    client_id: str = None
    server: str = "all"
    lease_time: str = None
    block_access: bool = False
    allow_dual_stack_queue: bool = True
    always_broadcast: bool = False
    dhcp_option: str = None
    dhcp_option_set: str = None
    rate_limit: str = None
    parent_queue: str = None
    queue_type: str = None
    routes: str = None
    insert_queue_before: str = None
    address_lists: str = None
    comment: str = None
    disabled: bool = False
    # Active
    active_address: str = None
    active_client_id: str = None
    active_mac_address: str = None
    active_server: str = None
    blocked: bool = None
    dynamic: bool = None
    expires_after: str = None
    host_name: str = None
    last_seen: str = None
    radius: str = None
    status: str = None
    id: str = None
    copy_from: str = None

    def __bool__(self) -> bool:
        return not self.disabled



from attr import dataclass


@dataclass
class HotspotHost:
    id: str
    mac_address: str
    address: str
    to_address: str
    server: str
    idle_time: str = None
    uptime: str = None
    host_dead_time: str = None
    idle_timeout: str = None
    bridge_port: str = None
    vlan_id: int = None
    keepalive_timeout: str = None
    bytes_in: int = 0
    packets_in: int = 0
    bytes_out: int = 0
    packets_out: int = 0
    rx_rate: int = 0
    tx_rate: int = 0
    rx_packets: int = 0
    tx_packets: int = 0
    found_by: str = None
    authorized: bool = False
    DHCP: bool = False
    comment: str = None



from ros._base import BaseModule, BaseProps, BaseProp

from .forwarding_table import MPLSForwardingTable
from .interface import MPLSInterface
from .ldp import MPLSLDP, LDPInstance
from .settings import MPLSSettings


class MPLSModule(BaseModule):
    _forwarding_table: BaseProps[MPLSForwardingTable] = None
    _interface: BaseProps[MPLSInterface] = None
    _ldp: MPLSLDP = None
    _settings: BaseProp[MPLSSettings] = None

    @property
    def forwarding_table(self):
        if not self._forwarding_table:
            self._forwarding_table = BaseProps(
                self.ros, "/mpls/forwarding-table", MPLSForwardingTable
            )
            self._forwarding_table._create = False
            self._forwarding_table._delete = False
            self._forwarding_table._write = False
        return self._forwarding_table

    @property
    def interface(self):
        if not self._interface:
            self._interface = BaseProps(self.ros, "/mpls/interface", MPLSInterface)
        return self._interface

    @property
    def ldp(self):
        if not self._ldp:
            self._ldp = MPLSLDP(self.ros, "/mpls/ldp", LDPInstance)
        return self._ldp

    @property
    def settings(self):
        if not self._settings:
            self._settings = BaseProp(self.ros, "/mpls/settings", MPLSSettings)
        return self._settings



from attr import dataclass
from typing import Literal

RuleActions = Literal["drop", "lookup", "lookup-only-in-table", "unreachable"]


@dataclass
class RoutingRule:
    action: RuleActions
    table: str
    dst_address: str = None
    interface: str = None
    src_address: str = None
    min_prefix: str = None
    routing_mark: str = None
    comment: str = None
    disabled: bool = None
    id: str = None
    place_before: str = None
    copy_from: str = None



from typing import Literal

AllLiteral = Literal["all"]
AnyLiteral = Literal["any"]
ARPLiteral = Literal[
    "disabled", "enabled", "local-proxy-arp", "proxy-arp", "reply-only"
]
IPProtocol = Literal[
    "dccp",
    "ddp",
    "egp",
    "encap",
    "etherip",
    "ggp",
    "gre",
    "hmp",
    "icmp",
    "icmpv6",
    "idpr-cmtp",
    "igmp",
    "ipencap",
    "ipip",
    "ipsec-ah",
    "ipsec-esp",
    "ipv6-encap",
    "ipv6-frag",
    "ipv6-nonxt",
    "ipv6-opts",
    "ipv6-route",
    "iso-tp4",
    "l2tp",
    "ospf",
    "pim",
    "pup",
    "rdp",
    "rspf",
    "rsvp",
    "sctp",
    "st",
    "tcp",
    "udp",
    "udp-lite",
    "vmtp",
    "vrrp",
    "xns-idp",
    "xtp",
]

MACProtocol = Literal[
    "802.2",
    "arp",
    "homeplug-av",
    "ip",
    "ipv6",
    "ipx",
    "lldp",
    "loop-protect",
    "mpls-multicast",
    "mpls-unicast",
    "packing-compr",
    "packing-simple",
    "pppoe",
    "pppoe-discovery",
    "rarp",
    "service-vlan",
    "vlan",
]

PortLiteral = Literal[
    "acap",
    "activision",
    "afpovertcp",
    "agentx",
    "aol",
    "apple-licman",
    "appleqtc",
    "appleqtcsrvr",
    "appleugcontrol",
    "arcp",
    "asia",
    "asip-webadmin",
    "asipregistry",
    "aurp",
    "auth",
    "avt-profile-1",
    "avt-profile-2",
    "bdp",
    "bftp",
    "bgp",
    "biff",
    "bootpc",
    "bootps",
    "btserv",
    "buddyphone",
    "ccmail",
    "cfdptkt",
    "chargen",
    "cops",
    "cpq-wbem",
    "csnet-ns",
    "daytime",
    "dict",
    "discard",
    "discovery",
    "distributed-net",
    "dixie",
    "dlsrap",
    "dlsrpn",
    "dlswpn",
    "dns",
    "dns2go",
    "doom",
    "dsp",
    "dtspcd",
    "echo",
    "epmap",
    "eppc",
    "esro-emsdp",
    "esro-gen",
    "etftp",
    "fcp-addr-srvr1",
    "fcp-addr-srvr2",
    "fcp-cics-gw1",
    "fcp-srvr-inst1",
    "fcp-srvr-inst2",
    "finger",
    "ftp",
    "ftp-data",
    "glimpse",
    "gopher",
    "h323gatestat",
    "h323hostcall",
    "half-life",
    "hbci",
    "hostname",
    "hotsync-1",
    "hotsync-2",
    "hsrp",
    "htcp",
    "http",
    "http-alt",
    "https",
    "ica",
    "icabrowser",
    "icpv2",
    "icq",
    "imail-www",
    "imap3",
    "imaps",
    "ipp",
    "irc",
    "ircu",
    "isakmp",
    "iso-tsap",
    "kerberos",
    "klogin",
    "l2tp",
    "ldap",
    "ldaps",
    "link",
    "linuxconf",
    "liquidaudio",
    "lotusnote",
    "lpr",
    "mac-srvr-admin",
    "madcap",
    "matip-type-a",
    "matip-type-b",
    "mc-ftp",
    "mgcp-callagent",
    "mgcp-gateway",
    "microcom-sbp",
    "mobileip",
    "mountd",
    "mpp",
    "ms-rpc",
    "ms-sql-m",
    "ms-sql-s",
    "ms-streaming",
    "ms-wbt-server",
    "msbd",
    "msg-auth",
    "msg-icp",
    "msql",
    "mtp",
    "mysql",
    "mzap",
    "name",
    "napster",
    "napster-2",
    "napster-3",
    "net-assistant",
    "netbios-dgm",
    "netbios-ns",
    "netbios-ssn",
    "netrek",
    "netstat",
    "nextstep",
    "nfile",
    "nfs",
    "nicname",
    "nntp",
    "nntps",
    "ntp",
    "odette-ftp",
    "odmr",
    "oracle-sql",
    "pcanywheredata",
    "pcanywherestat",
    "pdap-np",
    "pgp5-key",
    "photuris",
    "pop2",
    "pop3",
    "pop3s",
    "poppassd",
    "pptp",
    "prospero-np",
    "pwdgen",
    "qotd",
    "quake",
    "quake-world",
    "quake3",
    "radius",
    "radius-acct",
    "rap",
    "re-mail-ck",
    "realsecure",
    "reftek",
    "rip",
    "ripng",
    "rje",
    "rlogin",
    "rlp",
    "rrp",
    "rsvp-tunnel",
    "rtp",
    "rtsp",
    "rwhois",
    "rwp",
    "secureid",
    "sftp",
    "sgmp",
    "sift-uft",
    "sip",
    "slmail",
    "smb",
    "smtp",
    "smux",
    "snmp",
    "snpp",
    "socks",
    "sql*net",
    "sql-net",
    "squid",
    "ssh",
    "statsrv",
    "sunrpc",
    "supdup",
    "swat",
    "syslog",
    "systat",
    "t.120",
    "tacacs",
    "talk",
    "tcp-id-port",
    "tcpmux",
    "telnet",
    "tftp",
    "tftp-mcast",
    "timbuktu",
    "timbuktu-srv1",
    "timbuktu-srv2",
    "timbuktu-srv3",
    "timbuktu-srv4",
    "time",
    "tinc",
    "tlisrv",
    "uls",
    "unreal",
    "uucp-path",
    "vemmi",
    "veracity",
    "virtualuser",
    "vmnet",
    "vnc-1",
    "vnc-2",
    "webobjects",
    "whois++",
    "winbox",
    "winbox-old",
    "winbox-old-tls",
    "wingate",
    "wlbs",
    "x11",
    "xdmcp",
    "yahoo",
    "z39.50",
]

TCPState = Literal[
    "close",
    "close-wait",
    "established",
    "fin-wait",
    "last-ack",
    "listen",
    "none",
    "syn-recv",
    "syn-sent",
    "time-wait",
]



from typing import List

from ros._base import BaseProps
from .dhcp_server import DHCPServer
from .lease import DHCPLease
from .network import DHCPNetwork


class DHCPServerModule(BaseProps[DHCPServer]):
    _lease: BaseProps[DHCPLease] = None
    _network: BaseProps[DHCPNetwork] = None

    @property
    def lease(self) -> BaseProps[DHCPLease]:
        if not self._lease:
            self._lease = BaseProps(self.ros, "/ip/dhcp-server/lease", DHCPLease)
        return self._lease

    @property
    def network(self) -> BaseProps[DHCPNetwork]:
        if not self._network:
            self._network = BaseProps(self.ros, "/ip/dhcp-server/network", DHCPNetwork)
        return self._network


__all__ = ["DHCPNetwork", "DHCPServerModule", "DHCPServer", "DHCPLease"]



from attr import dataclass


@dataclass(slots=True)
class Traceroute:
    section: int
    address: str
    avg: float
    best: float
    last: str
    loss: float
    sent: int
    status: str
    std_dev: float
    worst: float

    def __str__(self) -> str:
        return self.address



from typing import Any, List
from ros._base import BaseModule, BaseProp, BaseProps

from .aaa import UserAAA
from .active import UserActive
from .group import UserGroup
from .settings import UserSettings
from .ssh_keys import UserSSHKey
from .user import User


class UserModule(BaseModule):
    _aaa: BaseProp[UserAAA] = None
    _active: BaseProps[UserActive] = None
    _group: BaseProps[UserGroup] = None
    _settings: BaseProp[UserSettings] = None
    _ssh_keys: BaseProps[UserSSHKey] = None
    _user: BaseProps[User] = None

    @property
    def aaa(self) -> BaseProp[UserAAA]:
        if not self._aaa:
            self._aaa = BaseProp(self.ros, "/user/aaa", UserAAA)
        return self._aaa

    @property
    def active(self) -> BaseProps[UserActive]:
        if not self._active:
            self._active = BaseProps(self.ros, "/user/active", UserActive)
        return self._active

    @property
    def group(self) -> BaseProps[UserGroup]:
        if not self._group:
            self._group = BaseProps(self.ros, "/user/group", UserGroup)
        return self._group

    @property
    def settings(self) -> BaseProp[UserSettings]:
        if not self._settings:
            self._settings = BaseProp(self.ros, "/user/settings", UserSettings)
        return self._settings

    @property
    def ssh_keys(self) -> BaseProps[UserSSHKey]:
        if not self._ssh_keys:
            self._ssh_keys = BaseProps(self.ros, "/user/ssh-keys", UserSSHKey)
        return self._ssh_keys

    @property
    def user(self) -> BaseProps[User]:
        if not self._user:
            self._user = BaseProps(self.ros, "/user", User)
        return self._user

    def __call__(self, **kwds: Any) -> List[User]:
        return self.user(**kwds)



from attr import dataclass


@dataclass(slots=True)
class IPAddressList:
    list: str
    address: str
    comment: str = None
    disabled: bool = None
    timeout: str = None
    creation_time: str = None
    dynamic: bool = None
    id: str = None



from attr import dataclass
from typing import List, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from ros.ros import Ros

from .cache import DNSCache
from .static import DNSStatic


@dataclass
class DNS:
    allow_remote_requests: bool
    cache_max_ttl: str
    cache_size: int
    cache_used: int
    dynamic_servers: str
    max_concurrent_queries: int
    max_concurrent_tcp_sessions: int
    max_udp_packet_size: int
    query_server_timeout: str
    query_total_timeout: str
    servers: str
    use_doh_server: str
    verify_doh_cert: bool
    _ros: Optional["Ros"] = None

    def cache(self, **kwds) -> List[DNSCache]:
        assert self._ros
        return self._ros.get_as("/ip/dns/cache", List[DNSCache], kwds)

    def static(self, **kwds) -> List[DNSStatic]:
        assert self._ros
        return self._ros.get_as("/ip/dns/static", List[DNSStatic], kwds)

    def flush(self):
        assert self._ros
        self._ros.post_as("/ip/dns/cache/flush", List[DNSCache])


__all__ = ["DNSCache", "DNS", "DNSStatic"]



from attr import dataclass
from typing import Union

from ros._base import BaseProps
from .profile import HotspotUserProfile


@dataclass
class HotspotUser:
    name: str
    password: str = None
    server: str = "all"
    address: str = None
    mac_address: str = None
    profile: str = "default"
    routes: str = None
    email: str = None
    limit_uptime: str = None
    limit_bytes_in: Union[str, int] = None
    limit_bytes_out: Union[str, int] = None
    limit_bytes_total: Union[str, int] = None
    disabled: bool = False
    comment: str = None
    copy_from: str = None
    id: str = None
    dynamic: bool = None
    bytes_in: int = None
    bytes_out: int = None
    packets_in: int = None
    packets_out: int = None
    uptime: str = None

    def __str__(self):
        return self.name


class HotspotUserModule(BaseProps[HotspotUser]):
    _profile: BaseProps[HotspotUserProfile] = None

    @property
    def profile(self) -> BaseProps[HotspotUserProfile]:
        if not self._profile:
            self._profile = BaseProps(
                self.ros, "/ip/hotspot/user/profile", HotspotUserProfile
            )
        return self._profile


__all__ = ["HotspotUser", "HotspotUserProfile", "HotspotUserModule"]



from attr import dataclass
from typing import Literal


Scheduled = Literal[
    "scheduled for disable", "scheduled for enable", "scheduled for uninstall", ""
]


@dataclass
class Package:
    id: str
    build_time: str
    disabled: bool
    name: str
    version: str
    scheduled: Scheduled
